
######################
## Precept 6 R code ##
######################

## Load the LaLonde job training dataset
## from the Matching package
## install.packages("Matching")
library(Matching)
library(dplyr)
library(ggplot2)

data(lalonde)
data <- lalonde

## Look at the distribution of y
qplot(data$re78,geom = "histogram",bins = 15)

ggplot(data,
       aes(x = educ,
           y = re78)) + 
  geom_point()

## Bivariate regression
fit <- lm(re78 ~ educ,
          data = data)

## Look at the output. What is the
## null hypothesis,
## test statistic,
## p-value

summary(fit)
t.stat <- summary(fit)$coefficients[,"Estimate"] / 
  summary(fit)$coefficients[,"Std. Error"]
2*pt(-abs(t.stat), df = nrow(data) - length(coef(fit)))

## Get predicted values
pred <- predict(fit, interval = "confidence")
data <- cbind(data,pred)

## Plot the regression line and confidence interval
ggplot(data,
       aes(x = educ, y = fit,
           ymin = lwr, ymax = upr)) +
  geom_line() + 
  geom_ribbon(alpha = .2) + 
  geom_point(aes(x = educ, y = re78))

## Why is the ribbon smaller in the middle?

## Plot residuals vs. fitted values
data$residuals <- residuals(fit)

ggplot(data,
       aes(x = fit, y = residuals)) +
  geom_point() +
  stat_smooth(se = FALSE, color = "red")

## Should we be worried?
## Heteroskedasticity: standard errors might be biased
## Non-linearity: coefficients might be biased

## Regression with two variables
fit2 <- lm(re78 ~ educ + age,
           data = data)
summary(fit2)

## Get predictions and confidence interval
data$fit2 <- predict(fit2)

## Plot the age-earnings association, by education
ggplot(data,
       aes(x = age, y = fit2,
           color = factor(educ))) +
  geom_line()


## Can we calculate those coefficients by hand?
## 1. Get the residuals of age on education
## 2. Regress earnings on those residuals
resid1 <- residuals(lm(age ~ educ, data = data))
our.est <- coef(lm(re78 ~ resid1, data = data))
our.est
coef(fit2)

## Plot what we just did
ggplot(data,
       aes(x = resid1, y = re78)) +
  geom_point() +
  geom_abline(aes(intercept = our.est[1],
                  slope = our.est[2])) +
  annotate("text", x = -4, y = 20000,
           label = paste("Slope =",round(our.est[2],2)))

## We can also repeat for the other coefficient
resid1 <- residuals(lm(educ ~ age, data = data))
coef(lm(re78 ~ resid1, data = data))
coef(fit2)

## What if we wanted to interact education and age?
fit3 <- lm(re78 ~ educ + age + educ:age,
           data = data)
## Predict values to plot
data$fit3 <- predict(fit3)
ggplot(data,
       aes(x = age, y = fit3,
           color = factor(educ))) +
  geom_line()

## Now look at residual plot
data$resid3 <- residuals(fit3)
ggplot(data,
       aes(x = fit3, y = resid3)) +
  geom_point() +
  stat_smooth(se = FALSE, color = "red")

## DISCUSS what would have happened if failed to include
## lower-order terms

## What if there is measurement error in X?
coef(fit)
cov(data$educ,data$re78) / var(data$educ)

educ.noise <- rnorm(nrow(data))
cov(data$educ + educ.noise,data$re78) / var(data$educ + educ.noise)

## Correcting for error by dividing by the
## proportion of the variance that is observed
coef(fit)
ggplot(data.frame(prop.true = seq(.8,1,.01),
                  beta.est = coef(fit)[2] / seq(.8,1,.01)),
       aes(x = prop.true, y = beta.est)) +
  geom_line()


## More fun with residual plots
set.seed(08544)
x <- 10*rbeta(1000,.3,.3)
rand.half <- rbinom(1000,1,.5)
x[x>1 & x<9 & rand.half==1] <- 5
rand.y <- runif(1000)
rand.y.disc <- sample(c(6.5,9.5),1000,replace = T)
low.val <- rbinom(1000,1,.5)
u <- ifelse(x < 1, 10*rand.y,
            ifelse(x < 9, ifelse(rand.half == 1, 3,
                                 rand.y.disc + .5 * rand.y),
                   ifelse(low.val == 1, 1,
                          6.5 + 4 * rand.y)))
color <- factor(rbinom(1000,1,.5))
y <- 50*x - x ^ 2 + u
ggplot(data.frame(x = x, y = y, color = color),
       aes(x = x, y = y,
           color = color)) +
       geom_point() +
  scale_color_manual(values = c("orange","black"))

ggplot(data.frame(x = x, y = resid(lm(y ~ x)), color = color),
       aes(x = x, y = y,
           color = color)) +
  geom_point() +
  xlim(c(-5,15)) +
  scale_color_manual(values = c("orange","black"))

ggplot(data.frame(x = x, y = resid(lm(y ~ x + I(x ^ 2))), color = color),
       aes(x = x, y = y,
           color = color)) +
  geom_point() +
  xlim(c(-5,15)) +
  scale_color_manual(values = c("orange","black"))

## Could discuss the regression assumptions
## (lecture 5 slide 23)